// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_IndexedDelivery_hpp
#define anna_comm_IndexedDelivery_hpp

#include <anna/comm/Service.hpp>

namespace anna {

namespace comm {
class Resource;
}

namespace comm {

/**
   Servicio de reparto de peticiones. Selecciona el destino de la proxima peticion en base una clave numerica de forma
   que el recurso usado sera el que ocupe la posicicion i-esima. Donde i = key % N, donde key es la clave recibida
   y N es el numero total de recursos definidos en este reparto.
*/
class IndexedDelivery : public comm::Service {
public:
  /**
     Define las formas de actuar de comm::IndexedDelivery en caso de error.
  */
  struct Mode {
    enum _v {
      Strict, /**< Si el servicio seleccionado no esta disponible aborta el envio del mensaje */
      Flexible /**< Si el servicio seleccionado no esta disponible pasa al siguiente disponible */
    };
  };

  /**
     Constructor.
     \param name Nombre logico del servicio de reparto de carga.
     \param isCritical Indica las caracteristicas del reparto de carga. Debe valor \em true si el
     reparto de carga a crear es critico o \em false en otro caso.
     \param mode Modo de actuar en caso de que el recurso seleccionado no este disponible.
  */
  IndexedDelivery(const char* name, const bool isCritical, const Mode::_v mode) :
    comm::Service(name, isCritical),
    a_mode(mode),
    a_iikey(0)
  {;}

  /**
     Establece la clave que se usara para decidir el comm::Server al que se va a enviar el mensaje.
     \param key Clave usada para calcular el servidor destino.
     \warning Este metodo debe invocarse siempre antes de enviar un mensaje.
  */
  void prepare(const int key) throw(RuntimeException);

  std::string asString() const throw();

  xml::Node* asXML(xml::Node* parent) const throw();

  /**
     Metodo que devuelve el nombre completo de este selector de recursos.
     Para evitar ambiguedades este nombre incluye la lista completa de \em namespaces
     a los que pertenece la clase.
     \return Una cadena con el nombre de este selector.
  */
  static const char* className() throw() { return "anna::comm::IndexedDelivery"; }

private:
  //------------------------------------------------------------------------------------
  // - a_iiserver: iterador que apunta al servidor que debemos usar.
  //------------------------------------------------------------------------------------
  const Mode::_v a_mode;
  iterator a_iikey;

  virtual void do_initialize() throw(RuntimeException) {;}
  comm::Resource* do_apply() throw(RuntimeException);
};

}
}

#endif

