// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_TextVariable_hpp
#define anna_core_util_TextVariable_hpp

#include <anna/core/util/Variable.hpp>
#include <anna/core/util/String.hpp>

namespace anna {

class DataBlock;

class TextComposer;

/**
 * Variable que contiene los valores sobre los que trabaja el util::TextComposer.
 */
class TextVariable : public Variable {
public:
  /**
   * Asignacin a entero.
   * \param value Valor a asignar a esta variable.
   */
  TextVariable& operator= (const int value) throw(RuntimeException) { setValue(value); return *this; }

  /**
   * Asignacin a cadena.
   * \param value Valor a asignar a esta variable.
   */
  TextVariable& operator= (const char* value) throw(RuntimeException) { setValue(value); return *this; }

  /**
   * Asignacin a cadena.
   * \param value Valor a asignar a esta variable.
   */
  TextVariable& operator= (const anna::String& value) throw(RuntimeException) { setValue(value.c_str()); return *this; }

  /**
   * Asignacin a entero de 64 bits.
   * \param value Valor a asignar a esta variable.
   */
  TextVariable& operator= (const S64 value) throw(RuntimeException) { setValue(value); return *this; }

  /**
   * Asignacin a flotante.
   * \param value Valor a asignar a esta variable.
   */
  TextVariable& operator= (const float value) throw(RuntimeException) { setValue(value); return *this; }

  /**
   * Asignacin a flotante.
   * \param value Valor a asignar a esta variable.
   */
  TextVariable& operator= (const double value) throw(RuntimeException) { setValue(value); return *this; }

protected:
  /**
   * Constructor.
   * \param name Nombre de la variable.
   * \param type Tipo de dato de la variable.
   * \param expression Expresin asociada a esta variable.
   */
  TextVariable(const char* name, const Type::_v type, const anna::String& expression);


private:
  const anna::String a_expression;

  const char* compose(DataBlock& buffer) const throw(RuntimeException);

  friend class TextComposer;
};

}

#endif

