// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_ldap_ResultCode_hpp
#define anna_ldap_ResultCode_hpp

#include <string>

namespace anna {

namespace ldap {

class Session;

/**
   Modela el resultado de las peticiones enviadas a servidores LDAP.

   \see ldap::Response
   \see ldap::Session::eventResponse
*/
class ResultCode {
public:
  /**
     Contructor.
  */
  ResultCode() : a_value(0) {;}

  /**
     Devuelve \em true si la respuesta recibida es correcta o \em false en otro caso.
     \return \em true si la respuesta recibida es correcta o \em false en otro caso.
  */
  bool isOk() const throw() { return a_value == 0; }

  /**
     Devuelve \em true si la respuesta recibida es un error debido a que las credenciales del usuario
     no son vlidas para conectar al servidor o \em false en otro caso.

     \return  \em true si la respuesta recibida es un error debido a que las credenciales del usuario
     no son vlidas para conectar al servidor o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isInvalidCredential() const throw();

  /**
     Devuelve \em true si la resulta recibida es LDAP_OPERATIONS_ERROR o \em false en otro caso.

     \return  \em true si la resulta recibida es LDAP_OPERATIONS_ERROR o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isOperationsError() const throw();

  /**
     Devuelve \em true si la resulta recibida es LDAP_PROTOCOL_ERROR o \em false en otro caso.

     \return  \em true si la resulta recibida es LDAP_PROTOCOL_ERROR o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isProtocolError() const throw();

  /**
     Devuelve \em true si la resulta recibida es LDAP_TIMELIMIT_EXCEEDED o \em false en otro caso.

     \return  \em true si la resulta recibida es LDAP_TIMELIMIT_EXCEEDED o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isTimeLimitExceeded() const throw();

  /**
     Devuelve \em true si la resulta recibida es LDAP_SIZELIMIT_EXCEEDED o \em false en otro caso.

     \return  \em true si la resulta recibida es LDAP_SIZELIMIT_EXCEEDED o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isSizeLimitExceeded() const throw();

  /**
     Devuelve \em true si la resulta recibida es LDAP_AUTH_METHOD_NOT_SUPPORTED o \em false en otro caso.

     \return  \em true si la resulta recibida es LDAP_AUTH_METHOD_NOT_SUPPORTED o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isAuthMethodNotSupported() const throw();

  /**
     Devuelve \em true si la resulta recibida es LDAP_STRONG_AUTH_REQUIRED o \em false en otro caso.

     \return  \em true si la resulta recibida es LDAP_STRONG_AUTH_REQUIRED o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isStrongAuthRequired() const throw();

  /**
     Devuelve \em true si la resulta recibida es LDAP_SASL_BIND_IN_PROGRESS o \em false en otro caso.

     \return  \em true si la resulta recibida es LDAP_SASL_BIND_IN_PROGRESS o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isSASLBindInProgress() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_ATTR_ERROR o \em false en otro caso,
     Puede darse al usar atributos no definidos, tipos inapropiados, comparaciones entre tipos no convertibles
     o errores sintxticos en general.

     \return \em true si la resulta recibida es un error de tipo LDAP_ATTR_ERROR o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isAttrError() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_NAME_ERROR o \em false en otro caso.
     Puede dar al usar objetos o alias no definidos o usar una sinstaxis no vlida en el DN.

     \return \em true si la resulta recibida es un error de tipo LDAP_NAME_ERROR o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isNameError() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_SECURITY_ERROR o \em false en otro caso.
     Puede darse por una autorizacin inapropiada, por presentar una credencial invlida o por intentar acceder
     a informacin sin los privilegios suficientes.

     Incluye el valor del mtodo #isInvalidCredential.

     \return \em true si la resulta recibida es un error de tipo LDAP_SECURITY_ERROR o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isSecurityError() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_SERVICE_ERROR o \em false en otro caso.
     Puede darse cuando el servicio no est disponible.

     \return \em true si la resulta recibida es un error de tipo LDAP_SERVICE_ERROR o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isServiceError() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_TIMEOUT o \em false en otro caso.

     \return \em true si la resulta recibida es un error de tipo LDAP_TIMEOUT o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isTimeout() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_UNAVAILABLE o \em false en otro caso.

     \return \em true si la resulta recibida es un error de tipo LDAP_UNAVAILABLE o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isUnavailable() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_SERVER_DOWN o \em false en otro caso.

     \return \em true si la resulta recibida es un error de tipo LDAP_SERVER_DOWN o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isServerDown() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_LOCAL_ERROR o \em false en otro caso.

     \return \em true si la resulta recibida es un error de tipo LDAP_LOCAL_ERROR o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isLocalError() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_DECODING_ERROR o \em false en otro caso.

     \return \em true si la resulta recibida es un error de tipo LDAP_DECODING_ERROR o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isDecodingError() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_FILTER_ERROR o \em false en otro caso.

     \return \em true si la resulta recibida es un error de tipo LDAP_FILTER_ERROR o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isFilterError() const throw();

  /**
     Devuelve \em true si la resulta recibida es un error de tipo LDAP_CONNECT_ERROR o \em false en otro caso.

     \return \em true si la resulta recibida es un error de tipo LDAP_CONNECT_ERROR o \em false en otro caso.

     \warning El valor retornado por este mtodo slo tendr validez en caso de que  el mtodo #isOk
     haya retornado \em false.
  */
  bool isConnectError() const throw();

  /**
     Devuelve el codigo de error asociado a esta instancia.
     \return El codigo de error asociado a esta instancia.
  */
  int getValue() const throw() { return a_value; }

  /**
     Devuelve el texto explicativo del error asociado a esta instancia.
     \return El texto explicativo del error asociado a esta instancia.
  */
  const std::string& getText() const throw() { return a_text; }

  /**
     Establece el codigo de error asociado a esta instancia.
     \param value Codigo de error
  */
//   void setValue (const int value) throw () { a_value = value; }

  /**
     Establece el texto explicativo del error asociado a esta instancia.
     \param text Texto explicativo del error
  */
//   void setText (const std::string& text) throw () { a_text = text; }

  /**
     Inicializa el codigo de error y el texto de esta instancia.
  */
  void clear() throw() { a_value = 0; a_text.clear(); }

  /**
     Operador copia.
     \param other Instancia de la que copiar.
     \return La instancia de el mismo despues de copiar los valores.
  */
  ResultCode& operator= (const ResultCode& other) throw() {
    a_value = other.a_value;
    a_text = other.a_text;
    return *this;
  }

  /**
   * Operador de asignacin.
   * \param ldap_result Cdigo de resultado con el que iniciar esta instancia.
   * \warning Si el valor de \em ldap_result no fue obtenido como resultado de ejecutar alguna operacion LDAP
   * el resultado de este mtodo no est definido.
   */
  ResultCode& operator= (const int ldap_result) throw();

  bool operator == (const int n) const throw() { return a_value == n; }
  bool operator != (const int n) const throw() { return a_value != n; }
  bool operator < (const int n) const throw() { return a_value < n; }
  bool operator <= (const int n) const throw() { return a_value <= n; }

  /**
     Devuelve una cadena con la informacion relevante sobre esta instancia.
     \return Una cadena con la informacion relevante sobre esta instancia.
  */
  const std::string asString() const throw();

private:
  int a_value;
  std::string a_text;

  ResultCode(const int ldap_method_result);

  void setValue(const int ldap_method_result, const int ldap_method_error) throw();

  bool extractResultCode(const Session*) throw();

  friend class Session;
};

}
}

#endif

