// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/core/oam/Module.hpp>
#include <anna/core/oam/Handler.hpp>
#include <anna/core/oam/CounterScope.hpp>
#include <anna/core/functions.hpp>

#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceWriter.hpp>
#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/core/functions.hpp>

// Standard
#include <stdarg.h>



//------------------------------------------------------------------------------
//-------------------------------------------------------- Handler::alarmEvent()
//------------------------------------------------------------------------------
const anna::oam::alarm_data_t *anna::oam::Handler::alarmEvent(const anna::oam::Module *module, const char *textPreffix, const char *textSuffix, char textSeparator, bool activation, const int & type, va_list argList) const throw() {
  // OAM alarm
  const alarm_data_t *result = module->alarm(type);

  // #define LOGINFORMATION(a) if (anna::Logger::isActive (anna::Logger::Information) == true) {a;}
  // Problemas con las comas dentro de la macro:
  if(anna::Logger::isActive(anna::Logger::Information)) {
    // Base text:
    std::string base = result ? result->Description : module->getDefaultInternalAlarmDescription(type);
    // replace possible __s__ with <%s>, and __d__ with <%d>
    base = anna::functions::replace(base, "__s__", "<%s>");
    base = anna::functions::replace(base, "__d__", "<%d>");
    // Final text:
    std::string alarm;

    if(textPreffix != NULL) {
      alarm = textPreffix;
      alarm += textSeparator;
      alarm += base;
    } else
      alarm = base;

    if(textSuffix != NULL) {
      alarm += textSeparator;
      alarm += textSuffix;
    }

    static char cad_aux[1024];
    vsprintf(cad_aux, alarm.c_str(), argList);
    std::string trace = module->getName();
    trace += anna::functions::asString(" | Alarm %s event: %s", (activation ? "activation" : "cancellation"), cad_aux);
    trace += anna::functions::asString(" | Enum type on module: %d", type);

    if(result) {
      if(activation)
        trace += anna::functions::asString(" | ActivationId: %d", result->ActivationId);
      else
        trace += anna::functions::asString(" | CancellationId: %d", result->CancellationId);

      trace += anna::functions::asString(" | DynamicVariablesCSL: %s", result->DynamicVariablesCSL.c_str());
      trace += anna::functions::asString(" | ExternalId: %d", result->ExternalId);
    }

    anna::Logger::information(trace, ANNA_FILE_LOCATION);
  }

  return result;
}


//------------------------------------------------------------------------------
//------------------------------------------------------ Handler::counterEvent()
//------------------------------------------------------------------------------

const anna::oam::counter_data_t *anna::oam::Handler::counterEvent(const anna::oam::Module *module, const int & type, const int & amount) const throw() {
  // OAM counter
  const counter_data_t *result = module->counter(type);
  // Base text:
  std::string base = result ? result->Description : module->getDefaultInternalCounterDescription(type);

  // Count if registered
  if(result) {
    anna::oam::CounterScope *nc_cs = const_cast <anna::oam::CounterScope*>(module->getActiveCounterScope());
    nc_cs->increment(result->Offset, amount);
  }

  LOGDEBUG
  (
    std::string trace = module->getName();
    trace += anna::functions::asString(" | Counter event: %s", base.c_str());
    trace += anna::functions::asString(" | Enum type on module: %d", type);
    trace += anna::functions::asString(" | Amount increased: %d", amount);

    if(result) trace += anna::functions::asString(" | Accumulated amount: %llu", module->getActiveCounterScope()->getAccValue(result->Offset));
    anna::Logger::debug(trace, ANNA_FILE_LOCATION);
  );

  return result;
}


