// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/util/Tokenizer.hpp>
#include <anna/core/functions.hpp>
#include <anna/config/defines.hpp>

// temporary
#include <iostream>

using namespace std;
using namespace anna;

//static
const int Tokenizer::MaxItem = 64;


Tokenizer::Tokenizer() :
  a_dataBlock(true),
  a_activateStrip(false) {
  a_items = new char* [MaxItem];
  anna_memset(a_items, 0, sizeof(char*) * MaxItem);
  a_maxItem = 0;
}

Tokenizer::Tokenizer(const char* str, const char* separator) :
  a_dataBlock(true),
  a_activateStrip(false) {
  a_items = new char* [MaxItem];
  anna_memset(a_items, 0, sizeof(char*) * MaxItem);
  a_maxItem = 0;

  try {
    apply(str, separator);
  } catch(Exception&) {
  }
}

Tokenizer::Tokenizer(const string& str, const char* separator) :
  a_dataBlock(true),
  a_activateStrip(false) {
  a_items = new char* [MaxItem];
  anna_memset(a_items, 0, sizeof(char*) * MaxItem);

  try {
    apply(str.c_str(), separator);
  } catch(Exception&) {
  }
}

Tokenizer::~Tokenizer() {
  delete [] a_items;
}

int Tokenizer::_apply(const char* str, const char* separator)
throw(RuntimeException) {

  a_maxItem = 0;

  //if(str == NULL)
  //  return 0;

  DataBlock mb(str, anna_strlen(str) + 1, false);
  a_dataBlock = mb;
  const char* buffer = a_dataBlock.getData();
  char *token = const_cast <char*>(buffer);
  char* last;

  while((token = strtok_r(token, separator, &last)) != NULL) {
    if(a_activateStrip == true)
      token = strip(token);

    a_items [a_maxItem ++] = token;
    token = NULL;

    if(a_maxItem == MaxItem) {
      string msg("Tokenizer::apply | String: ");
      msg += str;
      msg += " | Separator: ";
      msg += separator;
      msg += " | Tokenizer has not enough space";
      throw RuntimeException(msg, ANNA_FILE_LOCATION);
    }
  }

  return a_maxItem;
}

int Tokenizer::apply(const char *str, const char* separator, const char *tokenizeContiguous) throw(RuntimeException) {

  if(str == NULL)
    return 0;

  if (!separator)
    throw RuntimeException("Cannot tokenize with a NULL separator", ANNA_FILE_LOCATION);

  if (!tokenizeContiguous) return _apply(str, separator);

  std::string _str = str;
  std::string _sep = separator;
  std::string _tok = tokenizeContiguous;
  if (_sep == _tok)
    throw RuntimeException("Using the separator as artifial token is a nonsense (original behaviour)", ANNA_FILE_LOCATION);
  if (_tok == "")
    throw RuntimeException("Use another artifial token. Empty is a nonsense (original behaviour)", ANNA_FILE_LOCATION);

  std::string seps = _sep + _sep;
  std::size_t pos, sepsL = seps.size();
  std::string artificialToken = _sep + _tok + _sep;

  while ((pos = _str.find(seps)) != std::string::npos) _str.replace(pos, sepsL, artificialToken); 
  return _apply(_str.c_str(), separator);
}

const char* Tokenizer::at(const int i)
throw(RuntimeException) {
  if(i >= a_maxItem)
    indexException(i, ANNA_FILE_LOCATION);

  return data(begin() + i);
}

const char* Tokenizer::at(const int i) const
throw(RuntimeException) {
  if(i >= a_maxItem)
    indexException(i, ANNA_FILE_LOCATION);

  return data(begin() + i);
}

const char* Tokenizer::last() const
throw(RuntimeException) {
  if(a_maxItem == 0)
    throw RuntimeException("There is any token to select", ANNA_FILE_LOCATION);

  return data(begin() + a_maxItem - 1);
}

char* Tokenizer::strip(char* str)
throw() {
  char* result(str);

  if(str != NULL) {
    while(*result != 0 && *result == ' ')
      result ++;

    if(*result != 0) {
      char* final = result + anna_strlen(result) - 1;

      while(final > result && *final == ' ') {
        *final = 0;
        final --;
      }
    }
  }

  return result;
}

void Tokenizer::indexException(const int index, const char* fromFile, const int fromLine) const
throw(RuntimeException) {
  string msg(functions::asString("Index %d out of range [0,%d] | Items: ", index, a_maxItem));

  for(const_iterator ii = begin(), maxii = end(); ii != maxii; ii ++) {
    msg += data(ii);
    msg += ' ';
  }

  throw RuntimeException(msg, fromFile, fromLine);
}
