// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Includes propios
#include <anna/diameter/stack/Format.hpp>
#include <anna/diameter/stack/Avp.hpp>
#include <anna/diameter/functions.hpp>
#include <anna/diameter/stack/Dictionary.hpp>

#include <anna/core/functions.hpp>
#include <anna/xml/xml.hpp>

#include <anna/core/functions.hpp>

// STL
#include <string>

#include <anna/core/tracing/Logger.hpp>


//using namespace diameter::stack;

anna_assign_enum(anna::diameter::stack::Avp::FlagRule) = { "must", "may", "shouldnot", "mustnot", NULL /* list end indicator */};

//------------------------------------------------------------------------------
//------------------------------------------------------------------ Avp::~Avp()
//------------------------------------------------------------------------------
anna::diameter::stack::Avp::~Avp(void) {
}


//------------------------------------------------------------------------------
//--------------------------------------------------------------- Avp::isChild()
//------------------------------------------------------------------------------
bool anna::diameter::stack::Avp::isChild(const AvpId & avpId) const throw() {
  const Format * format = getFormat();

  if(!format->isGrouped()) return false;

  for(const_avprule_iterator it = avprule_begin(); it != avprule_end(); it++)
    if(avpId == ((*it).second.getId()))
      return true;

  return false;
}


//------------------------------------------------------------------------------
//--------------------------------------------------- Avp::getFlagsDescription()
//------------------------------------------------------------------------------
std::string anna::diameter::stack::Avp::getFlagsDescription(void) const throw() {
  std::string trace;
  trace += "V: "; trace += Avp::FlagRule::asText(a_vBit);
  trace += ", M: "; trace += Avp::FlagRule::asText(a_mBit);
  trace += ", P: "; trace += Avp::FlagRule::asText(a_pBit);
  return (trace);
}


//------------------------------------------------------------------------------
//----------------------------------------------- Avp::getFlagRulesDescription()
//------------------------------------------------------------------------------
std::string anna::diameter::stack::Avp::getFlagRulesDescription(void) const throw() {
  std::string trace;
  // Flag Rules:
  std::string s_must, s_may, s_shouldnot, s_mustnot, s_mayEncypt;
  s_must = (a_vBit == FlagRule::must) ? "V" : "";
  s_must += (a_mBit == FlagRule::must) ? ",M" : "";
  s_must += (a_pBit == FlagRule::must) ? ",P" : "";

  if(s_must == "") s_must = "-";

  if(s_must[0] == ',') s_must.erase(0, 1);

  s_may = (a_vBit == FlagRule::may) ? "V" : "";
  s_may += (a_mBit == FlagRule::may) ? ",M" : "";
  s_may += (a_pBit == FlagRule::may) ? ",P" : "";

  if(s_may == "") s_may = "-";

  if(s_may[0] == ',') s_may.erase(0, 1);

  s_shouldnot = (a_vBit == FlagRule::shouldnot) ? "V" : "";
  s_shouldnot += (a_mBit == FlagRule::shouldnot) ? ",M" : "";
  s_shouldnot += (a_pBit == FlagRule::shouldnot) ? ",P" : "";

  if(s_shouldnot == "") s_shouldnot = "-";

  if(s_shouldnot[0] == ',') s_shouldnot.erase(0, 1);

  s_mustnot = (a_vBit == FlagRule::mustnot) ? "V" : "";
  s_mustnot += (a_mBit == FlagRule::mustnot) ? ",M" : "";
  s_mustnot += (a_pBit == FlagRule::mustnot) ? ",P" : "";

  if(s_mustnot == "") s_mustnot = "-";

  if(s_mustnot[0] == ',') s_mustnot.erase(0, 1);

  s_mayEncypt = a_mayEncrypt ? "Y" : "N";

  if(s_must != "-") { trace += "must("; trace += s_must; trace += "), "; }

  if(s_may != "-") { trace += "may("; trace += s_may; trace += "), "; }

  if(s_shouldnot != "-") { trace += "shouldnot("; trace += s_shouldnot; trace += "), "; }

  if(s_mustnot != "-") { trace += "mustnot("; trace += s_mustnot; trace += "), "; }

  if(s_mayEncypt != "-") { trace += "mayEncypt("; trace += s_mayEncypt; trace += "), "; }

  if(trace != "") trace.erase(trace.size() - 2, 2);  // remove ', '

  return (trace);
}


//------------------------------------------------------------------------------
//------------------------------------------------------------- Avp::getFormat()
//------------------------------------------------------------------------------
const anna::diameter::stack::Format * anna::diameter::stack::Avp::getFormat() const throw() {
  return a_dictionary->getFormat(a_formatName);
}


//------------------------------------------------------------------------------
//-------------------------------------------------------------- Avp::asString()
//------------------------------------------------------------------------------
std::string anna::diameter::stack::Avp::asString() const throw() {
  std::string trace;
  //trace = "Avp '";
  trace = "'";
  trace += a_name;
  trace += "'";
  trace += anna::diameter::functions::avpIdAsPairString(a_id);
  trace += "|Format: ";
  const Format * format = getFormat();
  trace += format->asString();
  // Flag Rules:
  trace += "|FlagRules: ";
  trace += getFlagRulesDescription();

  if(format->isEnumerated()) {
    trace += "|Allowed enum values: "; trace += getEnums();
  }

  if(format->isGrouped()) {
    trace += "\n";

    for(const_avprule_iterator it = avprule_begin(); it != avprule_end(); it++) {
      // Align qualifier
      std::string qual = (*it).second.getQual();
      int NumberOfSpaces = strlen(DICTIONARY_AVPRULE_TAB) - qual.size();

      for(int k = 0; k < NumberOfSpaces; k++) trace += " ";

      trace += (*it).second.asString();
      trace += "\n";
    }
  } else {
    if(hasAliases()) {
      trace += "\n";

      for(const_label_iterator it = label_begin(); it != label_end(); it++) {
        trace += DICTIONARY_AVPRULE_TAB;
        trace += (*it).second;
        trace += " (";
        trace += (*it).first;
        trace += ")\n";
      }
    }
  }

  return (trace);
}


//------------------------------------------------------------------------------
//-------------------------------------------------------------- Avp::addLabel()
//------------------------------------------------------------------------------
void anna::diameter::stack::Avp::addLabel(const std::string & data,  const std::string & alias) throw(anna::RuntimeException) {
  const Format * format = getFormat();

  if(format->isGrouped())
    throw anna::RuntimeException("Cannot add 'data-alias' entry on a grouped Avp", ANNA_FILE_LOCATION);
  else
    a_labels[data] = alias;

  LOGWARNING(

  if(format->isEnumerated() && !allowEnum(atoi(data.c_str()))) {
  anna::Logger::warning(anna::functions::asString("Makes no sense adding alias '%s' for enum value '%s' out of range ('%s'), Avp '%s'",
                        alias.c_str(), data.c_str(), getEnums(), a_name.c_str()), ANNA_FILE_LOCATION);
  }
  );
}


//------------------------------------------------------------------------------
//------------------------------------------------------------ Avp::addAvpRule()
//------------------------------------------------------------------------------
void anna::diameter::stack::Avp::addAvpRule(const AvpRule & avpRule) throw(anna::RuntimeException) {
  const Format * format = getFormat();

  if(format->isGrouped()) {
    if(avpRule.isFixed()) {
      if(!a_allowFixedRule) {
        std::string s_ex = anna::functions::asString("Incorrect position for fixed avp rule '<%s>' within grouped avp '%s'", avpRule.getAvpName().c_str(), getName().c_str());
        s_ex += ". Fixed avp rules must be located at the beginning";
        throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
      }
    } else a_allowFixedRule = false;

    //a_avprules[avpRule.getId()] = avpRule;

    // Restriction for redefinition (at this same level) of two rules for the same avp:
    if(isChild(avpRule.getId())) {
      std::string s_ex = anna::functions::asString("Cannot add two rules for avp '%s', at the same level within grouped avp '%s'", avpRule.getAvpName().c_str(), getName().c_str());
      throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
    }

    a_avprules[a_avprulePosition++] = avpRule;
  } else
    throw anna::RuntimeException("Cannot add Avp rule on non-grouped Avp", ANNA_FILE_LOCATION);
}


//------------------------------------------------------------------------------
//----------------------------------------------------------------- Avp::asXML()
//------------------------------------------------------------------------------
anna::xml::Node* anna::diameter::stack::Avp::asXML(anna::xml::Node* parent) const throw() {
//   <!ELEMENT avp (single | grouped)>
//   <!ATTLIST avp name CDATA #REQUIREDcode CDATA #REQUIRED vendor-name CDATA #IMPLIED may-encrypt (yes | no) #IMPLIED v-bit (must | may | shouldnot | mustnot) #IMPLIED m-bit (must | may | shouldnot | mustnot) #IMPLIED p-bit (must | may | shouldnot | mustnot) #IMPLIED>
//   <!ELEMENT single (label*)>
//   <!ATTLIST single format-name CDATA #REQUIRED enum CDATA #IMPLIED>
//   <!ELEMENT label EMPTY>
//   <!ATTLIST label data CDATA #REQUIRED alias CDATA #REQUIRED>
//   <!ELEMENT grouped (avprule+)>
  anna::xml::Node* result = parent->createChild("avp");
  result->createAttribute("name", a_name);
  result->createAttribute("code", anna::functions::asString(a_id.first));

  if(a_id.second != 0) result->createAttribute("vendor-name", a_vendorName);

  result->createAttribute("may-encrypt", a_mayEncrypt ? "yes" : "no");

  if(a_vBit != Avp::FlagRule::None) result->createAttribute("v-bit", Avp::FlagRule::asText(a_vBit));

  if(a_mBit != Avp::FlagRule::None) result->createAttribute("m-bit", Avp::FlagRule::asText(a_mBit));

  if(a_pBit != Avp::FlagRule::None) result->createAttribute("p-bit", Avp::FlagRule::asText(a_pBit));

  const Format * format = getFormat();

  if(format->isGrouped()) {
    anna::xml::Node* grouped = result->createChild("grouped");

    for(const_avprule_iterator it = avprule_begin(); it != avprule_end(); it++)
      (*it).second.asXML(grouped);
  } else {
    anna::xml::Node* single = result->createChild("single");
    single->createAttribute("format-name", a_formatName);

    if(format->isEnumerated()) {
      std::string enums = getEnums();

      if(enums != "") single->createAttribute("enum", enums);
    }

    if(hasAliases()) {
      anna::xml::Node* label;

      for(const_label_iterator it = label_begin(); it != label_end(); it++) {
        label = single->createChild("label");
        label->createAttribute("data", (*it).first);
        label->createAttribute("alias", (*it).second);
      }
    }
  }

  return result;
}

