// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //

#include <iostream>
#include <string>

#include <limits.h>

#include <anna/core/util/Tokenizer.hpp>
#include <anna/core/util/Configuration.hpp>
#include <anna/core/util/Environment.hpp>
#include <anna/core/functions.hpp>

#include <gtest/gtest.h>


using namespace std;
using namespace anna;


TEST(core, tokenizer) {
  anna::Tokenizer lst;
  lst.apply("In three words", " ");
  EXPECT_EQ(lst.size(), 3);
  anna::Tokenizer::const_iterator tok_iter(lst.begin());
  ASSERT_STREQ(anna::Tokenizer::data(tok_iter++), "In");
  ASSERT_STREQ(anna::Tokenizer::data(tok_iter++), "three");
  ASSERT_STREQ(anna::Tokenizer::data(tok_iter++), "words");
  EXPECT_EQ(tok_iter, lst.end());
  ASSERT_THROW(lst.at(10), anna::RuntimeException);
  lst.apply("In three words", ",");
  EXPECT_EQ(lst.size(), 1);
  lst.apply("", ",");
  EXPECT_EQ(lst.size(), 0);
  ASSERT_THROW(lst.last(), anna::RuntimeException);
//  std::string str = "";
//  for (int k = 0; k < 100; k++)
//    str += "x ";
//  ASSERT_THROW(lst.apply(str, " "), anna::RuntimeException);
}

TEST(core, functions_asString) {
  std::string msg = anna::functions::asString("One %s has %d legs. Two %s have %d legs", "horse", 4, "horses", 8);
  EXPECT_EQ(msg, "One horse has 4 legs. Two horses have 8 legs");
  char cad_aux[128];
  sprintf(cad_aux, "%d", 123);
  EXPECT_EQ(anna::functions::asString(123), cad_aux);
  unsigned long ul = 43200111UL;
  sprintf(cad_aux, "%lu", ul);
  EXPECT_EQ(anna::functions::asString(ul), cad_aux);
  unsigned long long ull = 4321000111ULL;
  sprintf(cad_aux, "%llu", ull);
  EXPECT_EQ(anna::functions::asString(ull), cad_aux);
  U64 u64 = ull;
  EXPECT_EQ(anna::functions::asString(u64), cad_aux);
  unsigned int ui = 1234567890U;
  sprintf(cad_aux, "%u", ui);
  EXPECT_EQ(anna::functions::asString(ui), cad_aux);
  float f = 123.34;
  double d = 3.1415;
  sprintf(cad_aux, "%4.2f", f);
  EXPECT_EQ(anna::functions::asString(f, "%4.2f"), cad_aux);
  sprintf(cad_aux, "%4.2f", d);
  EXPECT_EQ(anna::functions::asString(d, "%4.2f"), cad_aux);
}

TEST(core, configuration) {
  anna::Configuration conf;
  ASSERT_THROW(conf.load("missing_file.cnf"), anna::RuntimeException);
  ASSERT_NO_THROW(conf.load("test/core/example.cnf"));
  //[ property ]
  //thing = tshirt
  //size = 1
  //color = blue
  //[ owner ]
  //name = edu
  //address = madrid
  ASSERT_THROW(conf.getValue("property", "WRONG_VAR"), anna::RuntimeException);
  ASSERT_THROW(conf.getValue("WRONG_SECTION", "thing"), anna::RuntimeException);
  std::string value;
  EXPECT_EQ(value = conf.getValue("property", "thing"), "tshirt");
  EXPECT_EQ(conf.getIntegerValue("property", "size"), 1);
  EXPECT_EQ(std::string(conf.getValue("property", "color")), "blue");
  EXPECT_EQ(std::string(conf.getValue("owner", "name")), "edu");
  EXPECT_EQ(std::string(conf.getValue("owner", "address")), "madrid");
  conf.setDefaultValue("owner", "phone", "555 55 55");
  EXPECT_EQ(std::string(conf.getValue("owner", "phone")), "555 55 55");
  EXPECT_TRUE(!conf.getValue("owner", "phone", true)); // query value is NULL: 'true = strict' even with default value assigned
  EXPECT_TRUE(!conf.exists("owner", "phone"));
  EXPECT_TRUE(conf.exists("owner", "name"));
}

TEST(core, environment) {
  anna::Environment &env = anna::Environment::instantiate();
  env.initialize();
  ASSERT_THROW(env.getValue("WRONG_ENV_VAR", true), anna::RuntimeException); // true => exception if missing
  ASSERT_THROW(env.getValue(NULL), anna::RuntimeException);
  env.setVariable("TEST_VAR", "my test var value");
  EXPECT_EQ(env.getValue("TEST_VAR"), std::string("my test var value"));
  env.setVariable("TEST_VAR", "my new test var value", false);  // false: no overwritting
  EXPECT_EQ(env.getValue("TEST_VAR"), std::string("my test var value"));
  env.unsetVariable("TEST_VAR");
  EXPECT_EQ(env.getValue("TEST_VAR"), std::string(""));
  EXPECT_EQ(env.getValue("MISSING_ENV_VAR"), std::string(""));
}


int main(int argc, char **argv)
{
    testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}

